@@ -81,11 +81,15 @@ ELSE
 DEFINE CALLP11=Frontend
 ENDIF
 
+INCLUDE "server_ssl.ml"
+
 (* PKCS#11 functions debug variable *)
 let ref_pkcs_debug = ref 0
 let ref_daemonize_args = ref ""
+IFNDEF WITH_SSL THEN
 let libnames_config_ref = ref ""
 let filter_config_file_ref = ref ""
+ENDIF
 
 (* Debug helper *)
 (* This function prints the name of the calling function *)
@@ -772,430 +776,8 @@ let setup srv _ =
     ~proc_c_loadmodule: c_LoadModule
     srv
 
-(* WITH SSL *)
-IFDEF WITH_SSL THEN
-IFDEF WITHOUT_FILTER THEN
-let configure cf addr =
-  (* Handle filter passthrough for the specific C_LoadModule call *)
-  let filter_config_file =
-    try
-      Some (cf # string_param (cf # resolve_parameter addr "filter_config"))
-    with
-      | Not_found -> (None); in
-  if filter_config_file <> None
-  then
-  begin
-      let s = Printf.sprintf "CONFIGURATION: unused option 'filter_config' found in the server configuration file while the server has been compiled with filter passthrough!" in
-      Netplex_cenv.log `Info s;
-  end;
-  let libnames_config =
-    try
-      cf # string_param (cf # resolve_parameter addr "libnames")
-    with
-      | Not_found -> failwith "Required parameter libnames is missing! (server compiled with filter passthrough mode)!" in
-    libnames_config_ref := libnames_config;
-  let use_ssl =
-    try
-      cf # bool_param (cf # resolve_parameter addr "use_ssl")
-    with
-      | Not_found -> false in
-  match use_ssl with
-  | true ->
-      let cafile =
-        try
-          cf # string_param (cf # resolve_parameter addr "cafile")
-        with
-          | Not_found ->
-          failwith "Required parameter cafile is missing!" in
-      let certfile =
-        try
-          cf # string_param (cf # resolve_parameter addr "certfile")
-        with
-          | Not_found ->
-          failwith "Required parameter certfile is missing!" in
-      let certkey =
-        try
-          cf # string_param (cf # resolve_parameter addr "certkey")
-        with
-          | Not_found ->
-          failwith "Required parameter certkey is missing!" in
-      let cipher_suite =
-        try
-          Some (cf # string_param (cf # resolve_parameter addr "cipher_suite"))
-        with
-          | Not_found -> (None); in
-      (* PFS handling *)
-      let dh_params =
-        try
-          Some (cf # string_param (cf # resolve_parameter addr "dh_params"))
-        with
-          | Not_found -> (None); in
-      let ec_curve_name =
-        try
-          Some (cf # string_param (cf # resolve_parameter addr "ec_curve_name"))
-        with
-          | Not_found -> (None); in
-      if cipher_suite = None
-      then
-      begin
-          let s = Printf.sprintf "CONFIGURATION: you did not set any cipher_suite list, it will use the OpenSSL HIGH suites!" in
-          Netplex_cenv.log `Info s;
-      end;
-      (* Certificate verification depth *)
-      let verify_depth =
-        try
-          Some (cf # int_param (cf # resolve_parameter addr "verify_depth"))
-        with
-          | Not_found -> (None); in
-
-      (* DHE PFS handling *)
-      if dh_params = None
-      then
-      begin
-          let s = Printf.sprintf "CONFIGURATION: you did not set any dh_params list, PFS DHE suites disabled" in
-          Netplex_cenv.log `Info s;
-      end;
-      (* ECDHE PFS handling *)
-      if ec_curve_name = None
-      then
-      begin
-          let s = Printf.sprintf "CONFIGURATION: you did not set any ec_curve_name list, PFS ECDHE suites disabled" in
-          Netplex_cenv.log `Info s;
-      end;
-      let allowed_clients_cert_path =
-        try
-          Some (cf # string_param (cf # resolve_parameter addr "allowed_clients_cert_path"))
-        with
-          | Not_found -> (None); in
-      if allowed_clients_cert_path = None
-      then
-      begin
-          let s = Printf.sprintf "CONFIGURATION: you did not set any allowed_clients_cert_path, any client with a proper certificate will be accepted" in
-          Netplex_cenv.log `Info s;
-      end
-      else
-      begin
-        let path = (match allowed_clients_cert_path with Some x -> x | _ -> "") in
-        let check_dir = (try Sys.is_directory path with
-           _ -> false) in
-         if check_dir = false then
-           let s = Printf.sprintf "Error: forbidden client certificates folder %s does not exist!" path in
-           failwith s
-      end;
-        (use_ssl, cafile, certfile, certkey, cipher_suite, dh_params, ec_curve_name, verify_depth, allowed_clients_cert_path)
-  | false -> (use_ssl, "", "", "", None, None, None, None, None)
-ELSE
-let configure cf addr =
-  (* Handle configuration file for the filter *)
-  let libnames_config =
-    try
-      Some (cf # string_param (cf # resolve_parameter addr "libnames"))
-    with
-      | Not_found -> (None); in
-  if libnames_config <> None
-  then
-  begin
-      let s = Printf.sprintf "CONFIGURATION: unused option 'libnames' found in the server configuration file while the server has been compiled to use the filter module!" in
-      Netplex_cenv.log `Info s;
-  end;
-  let filter_config_file =
-    try
-      cf # string_param (cf # resolve_parameter addr "filter_config")
-    with
-      | Not_found -> failwith "Required parameter filter_config is missing! (this is a path to the filter configuration rules)" in
-    filter_config_file_ref := filter_config_file;
-  let use_ssl =
-    try
-      cf # bool_param (cf # resolve_parameter addr "use_ssl")
-    with
-      | Not_found -> false in
-  match use_ssl with
-  | true ->
-      let cafile =
-        try
-          cf # string_param (cf # resolve_parameter addr "cafile")
-        with
-          | Not_found ->
-          failwith "Required parameter cafile is missing!" in
-      let certfile =
-        try
-          cf # string_param (cf # resolve_parameter addr "certfile")
-        with
-          | Not_found ->
-          failwith "Required parameter certfile is missing!" in
-      let certkey =
-        try
-          cf # string_param (cf # resolve_parameter addr "certkey")
-        with
-          | Not_found ->
-          failwith "Required parameter certkey is missing!" in
-      let cipher_suite =
-        try
-          Some (cf # string_param (cf # resolve_parameter addr "cipher_suite"))
-        with
-          | Not_found -> (None); in
-      if cipher_suite = None
-      then
-      begin
-          let s = Printf.sprintf "CONFIGURATION: you did not set any cipher_suite list, it will use the OpenSSL HIGH suites!" in
-          Netplex_cenv.log `Info s;
-      end;
-      (* PFS handling *)
-      let dh_params =
-        try
-          Some (cf # string_param (cf # resolve_parameter addr "dh_params"))
-        with
-          | Not_found -> (None); in
-      let ec_curve_name =
-        try
-          Some (cf # string_param (cf # resolve_parameter addr "ec_curve_name"))
-        with
-          | Not_found -> (None); in
-      if cipher_suite = None
-      then
-      begin
-          let s = Printf.sprintf "CONFIGURATION: you did not set any cipher_suite list, it will use the OpenSSL HIGH suites!" in
-          Netplex_cenv.log `Info s;
-      end;
-
-      (* DHE PFS handling *)
-      if dh_params = None
-      then
-      begin
-          let s = Printf.sprintf "CONFIGURATION: you did not set any dh_params list, PFS DHE suites disabled" in
-          Netplex_cenv.log `Info s;
-      end;
-      (* ECDHE PFS handling *)
-      if ec_curve_name = None
-      then
-      begin
-          let s = Printf.sprintf "CONFIGURATION: you did not set any ec_curve_name list, PFS ECDHE suites disabled" in
-          Netplex_cenv.log `Info s;
-      end;
-      (* Certificate verification depth *)
-      let verify_depth =
-        try
-          Some (cf # int_param (cf # resolve_parameter addr "verify_depth"))
-        with
-          | Not_found -> (None); in
-
-      let allowed_clients_cert_path =
-        try
-          Some (cf # string_param (cf # resolve_parameter addr "allowed_clients_cert_path"))
-        with
-          | Not_found -> (None); in
-      if allowed_clients_cert_path = None
-      then
-      begin
-          let s = Printf.sprintf "CONFIGURATION: you did not set any allowed_clients_cert_path, any client with a proper certificate will be accepted" in
-          Netplex_cenv.log `Info s;
-      end
-      else
-      begin
-        let path = (match allowed_clients_cert_path with Some x -> x | _ -> "") in
-        let check_dir = (try Sys.is_directory path with
-           _ -> false) in
-         if check_dir = false then
-           let s = Printf.sprintf "Error: forbidden client certificates folder %s does not exist!" path in
-           failwith s
-      end;
-        (use_ssl, cafile, certfile, certkey, cipher_suite, dh_params, ec_curve_name, verify_depth, allowed_clients_cert_path)
-  | false -> (use_ssl, "", "", "", None, None, None, None, None)
-
-ENDIF
-(* FIXME: ocaml-ssl does not currently support setting up PFS and other DH ciphers, 
-          if DH, ECDH suites are asked, we have to inform the user about it's
-          non-availability and remove them from the list 
-   TODO: patches have been submitted to ocaml-ssl to support new cipher suites, so 
-         these might become mainstream soon. Until then, we keep using the unsupported suites 
-         list.
-*)
-let unsupported_suites = ref ["ECDHE-RSA-RC4-SHA"; "ECDHE-ECDSA-RC4-SHA"; "ECDH-RSA-RC4-SHA"; "ECDH-ECDSA-RC4-SHA"; "DH-DSS-AES256-GCM-SHA384"; "DH-RSA-AES256-GCM-SHA384"; "DH-RSA-AES256-SHA256"; "DH-DSS-AES256-SHA256"; "DH-RSA-AES256-SHA"; "DH-DSS-AES256-SHA"; "DH-RSA-CAMELLIA256-SHA"; "DH-DSS-CAMELLIA256-SHA"; "ECDH-RSA-AES256-GCM-SHA384"; "ECDH-ECDSA-AES256-GCM-SHA384"; "ECDH-RSA-AES256-SHA384"; "ECDH-ECDSA-AES256-SHA384"; "ECDH-RSA-AES256-SHA"; "ECDH-ECDSA-AES256-SHA"; "DH-DSS-AES128-GCM-SHA256"; "DH-RSA-AES128-GCM-SHA256"; "DH-RSA-AES128-SHA256"; "DH-DSS-AES128-SHA256"; "DH-RSA-AES128-SHA"; "DH-DSS-AES128-SHA"; "DH-RSA-CAMELLIA128-SHA"; "DH-DSS-CAMELLIA128-SHA"; "ECDH-RSA-AES128-GCM-SHA256"; "ECDH-ECDSA-AES128-GCM-SHA256"; "ECDH-RSA-AES128-SHA256"; "ECDH-ECDSA-AES128-SHA256"; "ECDH-RSA-AES128-SHA"; "ECDH-ECDSA-AES128-SHA"]
-
-
-(* We do not let OpenSSL fallback to ugly ciphers *)
-let exclude_bad_ciphers = ref "!aNULL:!eNULL:!EXPORT:!DES:!3DES:!MD5:!PSK:!RC4"
-
-
-(* Check if an element is in a list *)
-let check_element_in_suites_list the_list element =
-  (* Find the element *)
-  let found = try Some (List.find (fun a -> compare a element = 0) the_list) with
-  (* If not found, return false *)
-  Not_found -> (None) in
-  if found = None
-  then
-    (false)
-  else
-  begin
-    (* Notify the user that the suite he specified is unsupported *)
-    let s = Printf.sprintf "CONFIGURATION: the '%s' SSL cipher suite is currently *not* supported by OCaml OpenSSL bindings => it is *removed* from the cipher suites that will be used!" element in
-    Netplex_cenv.log `Info s;
-    (true)
-  end
-
-(* Filter the unsupported suites *)
-let filter_PFS_ciphers ciphers =
-  (* Split the string with : *)
-  let the_list = Str.split (Str.regexp ":") ciphers in
-  (* For each suite, check if it is unsupported, and don't keep it if this is the case *)
-  let new_list = List.filter (fun a -> check_element_in_suites_list !unsupported_suites a = false) the_list in
-  let new_ciphers = String.concat ":" new_list in
-  (new_ciphers)
-
-(* Filter the empty ciphers suite or the one only containing *)
-(* negative expressions                                      *)
-let check_negative_only_ciphers ciphers = 
-  (* Split the string with : *)
-  let the_list = Str.split (Str.regexp ":") ciphers in
-  let check = List.fold_left (fun boolean element -> if compare (Str.string_match (Str.regexp "!") element 0) false = 0 then false else boolean) true the_list in
-  (check)
-
-let check_empty_negative_only_suites ciphers =
-  if compare ciphers "" = 0 then
-  begin
-    (* Empty ciphers suite case *)
-    let ciphers =  String.concat ":" ["HIGH"; ciphers] in
-    let s = Printf.sprintf "CONFIGURATION: the cipher_suite list is empty => we will use the OpenSSL HIGH suites!" in
-    Netplex_cenv.log `Info s;
-    (ciphers)
-  end
-  else
-  begin
-    (* Check for the presence of negative only expressions *)
-    let check_neg = check_negative_only_ciphers ciphers in
-    if compare check_neg true = 0 then
-    begin
-      let ciphers =  String.concat ":" ["HIGH"; ciphers] in
-      let s = Printf.sprintf "CONFIGURATION: the cipher_suite list only contains negative expressions => we will append the OpenSSL HIGH suites!" in
-       Netplex_cenv.log `Info s;
-      (ciphers)
-    end
-    else
-      (* If there was no problem, just return the input ciphers *)
-      (ciphers)
-  end
-
-(* This function checks in the allowed_clients_cert_path folder if a given client *)
-(* is allowed                                                                     *)
-let read_file f =
-  let ic = open_in f in
-  let n = in_channel_length ic in
-  let s = String.create n in
-  really_input ic s 0 n;
-  close_in ic;
-  (s)
-
-let check_is_client_certificate_allowed allowed_clients_cert_path client_cert =
-  match allowed_clients_cert_path with
-     None -> true
-   | Some path ->
-     (* Go through all the client certificates in the path *)
-     let check_dir = (try Sys.is_directory path with
-       _ -> false) in
-     if check_dir = true then
-       (* List all files in the directory *)
-       let cert_files = Sys.readdir path in
-       (* Get the client certificate string *)
-       let tmp_file = Filename.temp_file "pkcs11proxy_server" "client_cert" in
-       let _ = Ssl.write_certificate tmp_file client_cert in
-       (* Read the cert file as a string *)
-       let client_cert_string = read_file tmp_file in
-       let check = ref false in
-       Array.iter (
-         fun file_name ->
-           let to_compare = (try read_file (path ^ Filename.dir_sep ^ file_name) with
-             _ ->  ""
-           )  in
-           if compare to_compare "" = 0 then
-             check := !check || false
-           else
-             if compare to_compare client_cert_string = 0 then
-               check := !check || true
-             else
-               check := !check || false
-       ) cert_files;
-       (!check)
-     else
-       let s = Printf.sprintf "Error: forbidden client certificates folder %s does not exist!" path in
-       failwith s
-
-let my_socket_config use_ssl cafile certfile certkey cipher_suite dh_params ec_curve_name verify_depth allowed_clients_cert_path =
-  match use_ssl with
-  | true ->
-    flush stdout;
-    Ssl.init();
-    let ctx = Ssl.create_context Ssl.TLSv1_2 Ssl.Server_context in
-    Ssl.set_verify ctx [ Ssl.Verify_peer; Ssl.Verify_fail_if_no_peer_cert ] None;
-
-    (* Setup given cipher_suite *)
-    begin
-    match cipher_suite with
-        None -> (let new_cipher = String.concat ":" ["HIGH"; !exclude_bad_ciphers] in 
-                            try 
-                                Ssl.set_cipher_list ctx new_cipher
-                            with
-                                _ -> let s = Printf.sprintf "Unsupported cipher suite when configuring OpenSSL" in
-                                                    failwith s)
-       | Some ciphers -> ( let new_ciphers = filter_PFS_ciphers ciphers in
-                           let new_ciphers = check_empty_negative_only_suites new_ciphers in
-                           let new_cipher = String.concat ":" [new_ciphers; !exclude_bad_ciphers] in 
-                            try 
-                                Ssl.set_cipher_list ctx new_cipher
-                            with
-                                _ -> let s = Printf.sprintf "Unsupported cipher list %s" ciphers in
-                                                    failwith s)
-    end;
-
-    Ssl.set_client_CA_list_from_file ctx cafile;
-    begin
-    match verify_depth with
-     None -> Ssl.set_verify_depth ctx 4;
-    | Some params -> Ssl.set_verify_depth ctx params;
-    end;
-
-    Ssl.load_verify_locations ctx cafile "" ;
-    Ssl.use_certificate ctx certfile certkey;
-
-    begin
-    match dh_params with
-     None -> ()
-    | Some params -> try Ssl.init_dh_from_file ctx params
-                        with _ -> let s = Printf.sprintf "Could not set DH params from file %s" params in
-                        failwith s
-    end;
-
-    begin
-    match ec_curve_name with
-     None -> ()
-    | Some params -> try Ssl.init_ec_from_named_curve ctx params
-                        with _ -> let s = Printf.sprintf "Could not set EC curve name %s" params in
-                        failwith s
-    end;
-
-    Rpc_ssl.ssl_server_socket_config
-      ~get_peer_user_name:(fun _ sslsock ->
-                   prerr_endline "get_peer_user_name";
-                   let cert = Ssl.get_certificate sslsock in
-                   let user = Ssl.get_subject cert in
-                   (* Check peer client certificate *)
-                   let is_client_allowed = check_is_client_certificate_allowed allowed_clients_cert_path cert in
-                   if is_client_allowed = false then
-                     let s = Printf.sprintf "Unsupported client certificate for user=%s" user in
-                     (* Close the socket and quit *)
-                     let _ = Ssl.shutdown sslsock in
-                     failwith s
-                   else
-                     prerr_endline ("user=" ^ user);
-                     Some user)
-        ctx
-    | false -> Rpc_server.default_socket_config
-
-let socket_config (use_ssl, cafile, certfile, certkey, cipher_suite, dh_params, ec_curve_name, verify_depth, allowed_clients_cert_path) =
-  my_socket_config use_ssl cafile certfile certkey cipher_suite dh_params ec_curve_name verify_depth allowed_clients_cert_path
-
-ELSE
 (* WITHOUT SSL *)
+IFNDEF WITH_SSL THEN
 
 let socket_config _ = Rpc_server.default_socket_config
 
